package sysinfo

import (
	"encoding/json"
	"path/filepath"
	"regexp"
	"strings"
)

// BlockDevice + Partition live with this collector.

type BlockDevice struct {
	Timestamp  string                  `json:"timestamp"`
	Smartctl   string                  `json:"smartctl"`
	LuksDump   string                  `json:"luksDump,omitempty"`
	Sfdisk     json.RawMessage         `json:"sfdisk"`
	Blkid      string                  `json:"blkid"`
	Lsblk      json.RawMessage         `json:"lsblk"`
	Partitions map[string]*Partition   `json:"partitions"`
}

type Partition struct {
	Blkid string          `json:"blkid"`
	Lsblk json.RawMessage `json:"lsblk"`
}

func (a *App) collectBlock(s *Snapshot, now string) error {
	links, _ := filepath.Glob("/dev/disk/by-id/*")
	rePU := regexp.MustCompile(`PARTUUID="([^"]+)"`)
	seen := map[string]bool{}

	for _, link := range links {
		base := filepath.Base(link)
		target, _ := filepath.EvalSymlinks(link)
		dev := filepath.Base(target)
		if seen[dev] {
			continue
		}
		seen[dev] = true

		bd := &BlockDevice{
			Timestamp:  now,
			Partitions: map[string]*Partition{},
		}
		devPath := filepath.Join("/dev", dev)

		bd.Smartctl = a.safeRun("smartctl", "-a", devPath)
		if out := a.safeRun("cryptsetup", "luksDump", devPath); out != "" {
			bd.LuksDump = out
		}
		_ = json.Unmarshal(
			a.runJSON(&bd.Sfdisk, "sfdisk", "-J", devPath), &bd.Sfdisk)
		bd.Blkid = a.safeRun("blkid", "-p", devPath)
		_ = json.Unmarshal(
			a.runJSON(&bd.Lsblk, "lsblk", "-J", devPath), &bd.Lsblk)

		// enumerate partitions
		var ls struct {
			Blockdevices []struct {
				Children []struct{ Name string `json:"name"` } `json:"children"`
			} `json:"blockdevices"`
		}
		_ = json.Unmarshal(bd.Lsblk, &ls)
		for _, d := range ls.Blockdevices {
			for _, c := range d.Children {
				part := filepath.Join("/dev", c.Name)
				blk := a.safeRun("blkid", "-p", part)
				m := rePU.FindStringSubmatch(blk)
				if len(m) != 2 {
					continue
				}
				uuid := strings.ToLower(m[1])
				p := &Partition{Blkid: blk}
				_ = json.Unmarshal(
					a.runJSON(&p.Lsblk, "lsblk", "-J", part), &p.Lsblk)
				bd.Partitions[uuid] = p
			}
		}
		if s.Blockdevs == nil {
			s.Blockdevs = map[string]*BlockDevice{}
		}
		s.Blockdevs[base] = bd
	}
	return nil
}
