package sysinfo

import (
	"fmt"
	"os"
	"path/filepath"
)

// writeHierarchy mirrors the Snapshot into /etc/sysinfo.
func (a *App) writeHierarchy(s *Snapshot) error {
	if _, err := os.Stat(rootDir); err == nil && !a.force {
		return fmt.Errorf("%s exists; use --force", rootDir)
	}
	_ = os.RemoveAll(rootDir)
	if err := os.MkdirAll(rootDir, 0755); err != nil {
		return err
	}
	a.mustWriteText(rootDir, "snapshot_time.txt", s.SnapshotTime)

	if err := a.writeSystem(
		filepath.Join(rootDir, "system"), s.System); err != nil {
		return err
	}

	if s.Sensors != nil {
		sDir := filepath.Join(rootDir, "sensors")
		if err := os.MkdirAll(sDir, 0755); err != nil {
			return err
		}
		a.mustWriteText(sDir, "sensors.txt", s.Sensors.Output)
	}

	for id, bd := range s.Blockdevs {
		if err := a.writeBlockDevice(
			filepath.Join(rootDir, "blockdevs", id), bd); err != nil {
			return err
		}
	}

	for mac, n := range s.Network {
		if err := a.writeNetIface(
			filepath.Join(rootDir, "network", mac), n); err != nil {
			return err
		}
	}

	for pool, z := range s.ZFS {
		zDir := filepath.Join(rootDir, "zfs", "pools", pool)
		if err := os.MkdirAll(zDir, 0755); err != nil {
			return err
		}
		a.mustWriteText(zDir, "timestamp.txt", z.Timestamp)
		a.mustWriteText(zDir, "status.txt", z.Status)
		a.mustWriteText(zDir, "get.txt", z.Get)
		a.mustWriteText(zDir, "zfs_list.txt", z.ZfsList)
	}

	if s.Packages != nil {
		pDir := filepath.Join(rootDir, "packages")
		if err := os.MkdirAll(pDir, 0755); err != nil {
			return err
		}
		a.mustWriteText(pDir, "dpkg_list.txt", s.Packages.Dpkg)
	}

	f, err := os.Create(filepath.Join(rootDir, snapshotJSON))
	if err != nil {
		return err
	}
	defer f.Close()
	return a.emitJSON(f, s)
}

/* ------------------------------------------------------------------ */
/*  helpers for leaf sections                                         */
/* ------------------------------------------------------------------ */

func (a *App) writeSystem(dir string, sys *SystemData) error {
	if err := os.MkdirAll(dir, 0755); err != nil {
		return err
	}
	a.mustWriteText(dir, "timestamp.txt", sys.Timestamp)
	a.mustWriteText(dir, "id.txt", sys.ID)
	a.mustWriteText(dir, "distro.txt", sys.Distro)
	a.mustWriteText(dir, "lsb_release.txt", sys.LSBRel)
	a.mustWriteText(dir, "uname.txt", sys.Uname)
	a.mustWriteText(dir, "cpuinfo.txt", sys.CPUInfo)
	a.mustWriteText(dir, "meminfo.txt", sys.MemInfo)
	if sys.Dmidecode != "" {
		a.mustWriteText(dir, "dmidecode.txt", sys.Dmidecode)
	}
	return nil
}

func (a *App) writeBlockDevice(dir string, bd *BlockDevice) error {
	if err := os.MkdirAll(
		filepath.Join(dir, "partitions"), 0755); err != nil {
		return err
	}
	a.mustWriteText(dir, "timestamp.txt", bd.Timestamp)
	a.mustWriteText(dir, "smartctl.txt", bd.Smartctl)
	if bd.LuksDump != "" {
		a.mustWriteText(dir, "luksDump.txt", bd.LuksDump)
	}
	a.mustWriteJSON(dir, "sfdisk.json", bd.Sfdisk)
	a.mustWriteText(dir, "blkid.txt", bd.Blkid)
	a.mustWriteJSON(dir, "lsblk.json", bd.Lsblk)
	for uuid, p := range bd.Partitions {
		pDir := filepath.Join(dir, "partitions", uuid)
		if err := os.MkdirAll(pDir, 0755); err != nil {
			return err
		}
		a.mustWriteText(pDir, "blkid.txt", p.Blkid)
		a.mustWriteJSON(pDir, "lsblk.json", p.Lsblk)
	}
	return nil
}

func (a *App) writeNetIface(dir string, n *NetIface) error {
	if err := os.MkdirAll(dir, 0755); err != nil {
		return err
	}
	a.mustWriteText(dir, "timestamp.txt", n.Timestamp)
	a.mustWriteText(dir, "iface.txt", n.Iface)
	a.mustWriteJSON(dir, "ip_addr.json", n.IPAddr)
	a.mustWriteText(dir, "link.txt", n.Link)
	if n.Ethtool != "" {
		a.mustWriteText(dir, "ethtool.txt", n.Ethtool)
	}
	a.mustWriteJSON(dir, "statistics.json", n.Stats)
	if len(n.IPInfo) > 0 {
		a.mustWriteJSON(dir, "ipinfo.json", n.IPInfo)
	}
	return nil
}
